#!/usr/bin/env python
#
# Time-stamp: <99/11/08 17:42:37 hellmann>
#
"""

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation, and that the name of Doug
Hellmann not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


DESCRIPTION:

    A special kind of Folder which presents different views to
    different users.

CHANGE HISTORY:

$Log: Chameleon.py,v $
Revision 1.14  1999/11/08 22:43:50  hellmann
Cleanup of code.  This version actually seems to work as expected
in most cases.
Added two doc files that contain the same content as is present on line
on zope.org.

Revision 1.13  1999/11/08 21:06:49  hellmann
Removed dead code.

Revision 1.9  1999/11/08 20:34:24  hellmann
Continuing to try different schemes to get the behavior as desired.

Revision 1.8  1999/11/05 18:18:55  hellmann
Create a folder to hold shared content when the Chameleon is created.

Revision 1.7  1999/11/05 18:12:20  hellmann
Turned off tracing.

Revision 1.6  1999/11/05 13:45:03  hellmann
Added new demonstration (ChameleonDemonstration.zexp).
Fixed some problems with acquisition.
New icon.

Revision 1.5  1999/11/04 21:45:24  hellmann
The previous version did not work for objects other than the initial
requested object.  Once we had returned an object which did not belong
to us, the publisher could not find any other objects (at the same
level, or acquired).  This version resolves that problem if all of the
*shared* objects are stored in an intermediate folder.

Revision 1.4  1999/11/04 17:00:28  hellmann
Added helpful text to the DTML files for adding/editing products.
Added new icon files.

Revision 1.3  1999/11/03 20:22:30  hellmann
Changed tracing to pass through a trace function.  This function is now disabled by default.

Revision 1.2  1999/11/03 20:20:18  hellmann
Control behavior of Chameleon via cookies, as well as allowing the
site owner to set a default appearance.

Revision 1.1  1999/11/03 19:21:10  hellmann
Initial working version.

"""

#
#  Creation Information
#
__rcs_module_name__ = '$RCSfile: Chameleon.py,v $'
__creator__         = 'Doug Hellmann <doughellmann@home.com>'
__project__         = 'Open Source'
__created__         = 'Tue, 02-Nov-1999 16:39:37 EST'

#
#  Current Information
#
__author__          = '$Author: hellmann $'
__version__         = '$Revision: 1.14 $'
__date__            = '$Date: 1999/11/08 22:43:50 $'
__locker__          = '$Locker:  $'

#
# Import system modules
#
import sys

#
# Import Zope modules
#
from Globals import HTMLFile
import OFS
import Acquisition

#
# Import Local modules
#
from Skin import *

#
# Module
#

ADD_SKIN_PERMISSION = 'Add Chameleon Skin'

def trace(message):
    #sys.stderr.write(message)
    return


manage_addChameleonForm=HTMLFile('add', globals())

def manage_addChameleon(self, id, title='',
                        createDefaultSkin=0,
                        createUserF=0,
                        cookieName='',
                        sharedFolderName='',
                        REQUEST=None):
    """
    Create a new Chameleon Folder.
    """
    ob = Chameleon()
    ob.id = id
    ob.title = title
    ob.default_skin = DEFAULT_SKIN_NAME
    ob.cookie_name = cookieName
    self._setObject(id, ob)
    try: user=REQUEST['AUTHENTICATED_USER']
    except: user=None
    #
    # Create a user manager
    #
    if createUserF:
        if (user is not None) and not (
            user.has_permission('Add User Folders', self)):
            raise 'Unauthorized', (
                  'You are not authorized to add User Folders.'
                  )
        ob.manage_addUserFolder()
    #
    # Create the default skin, if one was requested
    #
    if createDefaultSkin:
        if (user is not None) and not (
            user.has_permission(ADD_SKIN_PERMISSION, self)):
            raise 'Unauthorized', (
                  'You are not authorized to add Skins.'
                  )
        ob.manage_addChameleonSkin(
            id=DEFAULT_SKIN_NAME,
            title='Skin created by default.',
            createPublic=1,
            createUserF=createUserF,
            REQUEST=REQUEST)
    #
    # Create the folder where shared content will go
    #
    if (user is not None) and not(
        user.has_permission('Add Folders', self)):
        raise 'Unauthorized', ('You are not authorized to create Folders.')
    ob.manage_addFolder(id=sharedFolderName,
                        title=sharedFolderName,
                        createPublic=1,
                        createUserF=0,
                        REQUEST=REQUEST)
    #
    # Return the management interface for the calling object
    #
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

    
    
class ChameleonBase:
    """
    Base class for Chameleon objects.
    """
    
    meta_type='Chameleon'
    icon='misc_/Chameleon/chameleon'

    manage_editForm = HTMLFile('edit', globals())
    
    manage_options = OFS.Folder.Folder.manage_options[:1] + (
        { 'label':'Edit',
          'action':'manage_editForm',
          },
        ) + OFS.Folder.Folder.manage_options[1:]

    __ac_permissions__ = OFS.Folder.Folder.__ac_permissions__ + (
        (ADD_SKIN_PERMISSION, ['manage_addChameleonSkin'], ('Manager',)),
        )

    def __get_skin_name__(self, REQUEST):
        try:
            return REQUEST.cookies[self.cookie_name]
        except (KeyError, AttributeError):
            return self.default_skin
        
    def __getattr__(self, name):
        trace('GETATTR(%s): %s\n' % (self.id, name))
        try:
            return OFS.Folder.Folder.__getattr__(self, name)
        except:
            if name not in ( '_v_REQUEST', ):
                try:
                    skin_name = self.__get_skin_name__(self._v_REQUEST)
                    skin = self.__dict__[skin_name]
                    return getattr(skin, name)
                except KeyError:
                    trace('Chameleon GETATTR(%s): do not have skin %s\n' \
                          % (self.id, skin_name))
                    raise AttributeError(name)
                except AttributeError:
                    trace('Chameleon GETATTR(%s):  do not have %s\n' % (self.id, name))
                    raise AttributeError(name)
            else:
                raise AttributeError(name)
        else:
            trace('Chameleon GETATTR: %s - have it\n' % name)
        return

    def trace__bobo_traverse__(self, REQUEST, name=''):
        trace('\n\nChameleon TRAVERSE: URL: %s\n' % REQUEST.URL)
        trace('Chameleon TRAVERSE: from %s (%s) to %s\n' \
                         % (self.title_or_id(), self, name))
        #trace('COOKIES:\n')
        #trace('Chameleon TRAVERSE: Chain=%s\n' % str(self.aq_chain))
        #for n, v in REQUEST.cookies.items():
        #    trace('\t%s=%s\n' % (n, v))
        return
    
    def __bobo_traverse__(self, REQUEST, name=''):
        self.trace__bobo_traverse__(REQUEST, name)
        self._v_REQUEST = REQUEST
        #
        # Determine the name of the skin to use
        #
        #trace('Looking for skin cookie %s\n' % self.cookie_name)
        try:
            skin_name = REQUEST.cookies[self.cookie_name]
            #trace('\tSKIN NAME from cookie %s\n' % self.cookie_name)
        except AttributeError:
            # no value for cookie_name
            skin_name = None
        except KeyError:
            # no cookie set
            skin_name = None
        if not skin_name:
            skin_name = self.default_skin
            #trace('\tSKIN NAME from default value\n')
        #
        # Override Acquisition to look inside the current
        # skin for the object.
        #
        try:
            current_skin = getattr(self, skin_name)
        except AttributeError:
            # no such skin?
            current_skin = self
        #
        # Look for attributes defined in our instance
        #
        if self.__dict__.has_key(name):
            trace('\tfrom self.__dict__\n')
            obj = getattr(self, name)
            #trace('\tobj is %s (%s)\n' % (id(obj), obj))
            if hasattr(obj, '__of__'):
                #trace('\n\nbefore __of__:%s\n' % id(obj))
                obj = obj.__of__(current_skin)
                #trace(' after __of__:%s\n\n\n' % id(obj))
            return obj
        #
        # Look for management attributes
        #
        if (name[:7] == 'manage_') or (name[0] == '_'):
            trace('\tmanagement or special attribute\n')
            return getattr(self, name)
        else:
            if hasattr(current_skin, name):
                trace('\t from skin\n')
                obj = getattr(current_skin, name)
                #trace('\n\nbefore __of__:%s\n' % id(obj))
                obj = obj.__of__(self)
                #trace(' after __of__:%s\n\n\n' % id(obj))
                return obj
        #
        # If we get here, we were not able to get the value
        # from a skin, so fall back on Acquisition.
        #
        trace('\tfrom Acquisition\n')
        return getattr(self, name)

    def SkinIDs(self):
        return self.objectIds('ChameleonSkin')
    
    def manage_edit(self, default_skin=None, cookieName=None, REQUEST=None):
        """
        Change parameters to this Chameleon.
        """
        if default_skin:
            self.default_skin = default_skin
        if cookieName:
            self.cookie_name = cookieName
        return self.manage_editForm(self, REQUEST)


class Chameleon(ChameleonBase, OFS.Folder.Folder):
    """
    A special kind of Folder which presents different views to
    different users.
    """
    pass

