#!/usr/bin/env python
#
# Time-stamp: <99/11/08 17:43:18 hellmann>
#
"""

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation, and that the name of Doug
Hellmann not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


DESCRIPTION:

    Define a folder type which is can be used as a skin for a
    Chameleon folder.

CHANGE HISTORY:

$Log: Skin.py,v $
Revision 1.5  1999/11/08 22:43:50  hellmann
Cleanup of code.  This version actually seems to work as expected
in most cases.
Added two doc files that contain the same content as is present on line
on zope.org.

Revision 1.4  1999/11/05 18:00:33  hellmann
Changed Skin icon to an animated gif which changes color once every 2 minutes.

Revision 1.3  1999/11/05 13:45:03  hellmann
Added new demonstration (ChameleonDemonstration.zexp).
Fixed some problems with acquisition.
New icon.

Revision 1.2  1999/11/03 20:20:59  hellmann
Removed dead code.
Set the name of the default skin to be created when the owner creates
the Chameleon.

Revision 1.1  1999/11/03 19:21:10  hellmann
Initial working version.

"""

#
#  Creation Information
#
__rcs_module_name__ = '$RCSfile: Skin.py,v $'
__creator__         = 'Doug Hellmann <doughellmann@home.com>'
__project__         = 'Open Source'
__created__         = 'Wed, 03-Nov-1999 14:12:41 EST'

#
#  Current Information
#
__author__          = '$Author: hellmann $'
__version__         = '$Revision: 1.5 $'
__date__            = '$Date: 1999/11/08 22:43:50 $'
__locker__          = '$Locker:  $'

#
# Import system modules
#
import sys

#
# Import Zope modules
#
from Globals import HTMLFile
import OFS
import Acquisition

#
# Import Local modules
#


#
# Module
#

DEFAULT_SKIN_NAME='defaultskin'

manage_addChameleonSkinForm=HTMLFile('addSkin', globals())

def manage_addChameleonSkin(self, id, title='',
                        createPublic=0,
                        createUserF=0,
                        REQUEST=None):
    """
    Create a new Chameleon Skin.
    """
    ob = ChameleonSkin()
    ob.id = id
    ob.title = title
    self._setObject(id, ob)
    try: user=REQUEST['AUTHENTICATED_USER']
    except: user=None
    if createUserF:
        if (user is not None) and not (
            user.has_permission('Add User Folders', self)):
            raise 'Unauthorized', (
                  'You are not authorized to add User Folders.'
                  )
        ob.manage_addUserFolder()
    if createPublic:
        if (user is not None) and not (
            user.has_permission('Add Documents, Images, and Files', self)):
            raise 'Unauthorized', (
                  'You are not authorized to add DTML Documents.'
                  )
        ob.manage_addDTMLDocument(id='index_html', title='')
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


class ChameleonSkin(OFS.Folder.Folder):
    meta_type='ChameleonSkin'
    icon='misc_/Chameleon/chameleonskin'

#     def aq_acquire(self, name):
#         sys.stderr.write('Skin->Acquire(%s)\n' % name)
#         return Acquisition.Implicit.aq_acquire(self, name)

#     def __getattr__(self, name):
#         sys.stderr.write('Skin GETATTR(%s): %s\n' % (self.id, name))
#         if name == 'icon':
#             return 'misc_/Chameleon/chameleonskin_red'
#         else:
#             return OFS.Folder.Folder.__getattr__(self, name)

#     def __bobo_traverse__(self, REQUEST, name=''):
#         sys.stderr.write('Skin TRAVERSE: %s\n' % name)
#         sys.stderr.write('Skin TRAVERSE URL: %s\n' % REQUEST.URL)
#         return OFS.Folder.Folder.__bobo_traverse__(self, REQUEST, name)
